//     File:       ServiceUSBToolbox.h
// 
//     Contains:   SERVICE USB Device APIs.
// 
//     Version:    1.2
// 
//     Copyright:   2001-2008 by Boenig und Kallenbach oHG, all rights reserved.
// 
//		 Author:		 Martin Bestmann
// 
//     Bugs	:      For bug reports, please send an email to
//                 bk@bkohg.com


#ifndef	__ServiceUSBToolbox__
#define	__ServiceUSBToolbox__

#if __MACH__
#ifdef __MWERKS__
#include <CarbonCore/MacTypes.h>
#else
#include <CoreServices/CoreServices.h>
#endif // __MWERKS__
#endif // __MACH__

#ifdef __cplusplus
#define __DefaultParam(x) =x
#else
#define __DefaultParam(x)
#endif // __cplusplus

// -----------------------------------------------------------------------------------------------
//  error code that can be returned by and calls
// -----------------------------------------------------------------------------------------------

enum
{
	errServiceUSB_OutOfParamBlocks=13000,			// No USB control blocks available. There are too many request executing.
	errServiceUSB_Timeout,										// A call did not finish for some reasons: Timeout was reached.
	errServiceUSB_BadSync,										// A call was issued from the wrong interrupt level.
	errServiceUSB_USBDriverNotFound,					// Couldn't find the USB SERVICE driver. Probably it is not loaded.
	errServiceUSB_WrongPipeIndex,							// A wrong index for the read/write pipe was specified (valid values are 0, 1 and 2.
	errServiceUSB_MaxDevicesReached,					// The maximum of USB SERVICE devices that can be connected has been reached.
	errServiceUSB_DeviceNotFound,							// The device couldnot be found in the device table
	errServiceUSB_CFNil,											// A call to CoreFoundation returned nil
	errServiceUSB_NoConfigurationsError, 			// No configuration where found
	errServiceUSB_API_NotSupported						// The API call is not supported
};


// -----------------------------------------------------------------------------------------------
//  types and constants
// -----------------------------------------------------------------------------------------------

#define kServiceUSB_DefaultTimeout								60	 					// most calls should finish within 1 second...
#define kServiceUSB_WaitForEverTimeout						0xFFFFFFFF	 	// wait until the call finishes...
#define kServiceUSB_ReturnWriteCallImmediately		0							// will return immediately 

#define kServiceUSB_UseDefaultDeviceIdentifier		0							// if this is specfied for ServiceUSB_DeviceIdentifierRef then the first device is used
																																// You normally use this if you know that only one device is connected.

// The DeviceIdentifier is a unqiue ID that needs to be passed to each call if you want to talk to a device.
// Actually the DeviceIdentifier is the serialnumber of the device.
typedef struct ServiceUSB_opaqueDeviceIdentifier* ServiceUSB_DeviceIdentifierRef;

// ---------------------------------------------------------------------------------------------------------------
//  API functions to communicate with the device
// ---------------------------------------------------------------------------------------------------------------

#ifdef __cplusplus
extern "C" {
#endif

// ---------------------------------------------------------------------------------------------------------------
// ---------------------------------------------------------------------------------------------------------------
// --------------- API functions that read/write bytes and get the status of the output/input port ---------------
// ---------------------------------------------------------------------------------------------------------------
// ---------------------------------------------------------------------------------------------------------------

// ---------------------------------------------------------------------------------------------------------------
//  OSStatus ServiceUSB_SetOutputValue(UInt16 inValue,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier=kServiceUSB_UseDefaultDeviceIdentifier,UInt32 inTimeout=kServiceUSB_DefaultTimeout)
//  
//  Use this function to set the eight digital output lines(bits) either to high or low
//
//	inValue: the new value for the output port
//	inDeviceIdentifier: the DeviceIdentifier that identifies the device that's targeted
//	inTimeout: the time after which the call will return if the request could not be passed to the device.
//
//  Returns noErr if successfull.
// ---------------------------------------------------------------------------------------------------------------
OSStatus ServiceUSB_SetOutputValue(UInt16 inValue,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier __DefaultParam(kServiceUSB_UseDefaultDeviceIdentifier),UInt32 inTimeout __DefaultParam(kServiceUSB_DefaultTimeout));

// ---------------------------------------------------------------------------------------------------------------
//  UInt16 ServiceUSB_GetInputValue(ServiceUSB_DeviceIdentifierRef inDeviceIdentifier=kServiceUSB_UseDefaultDeviceIdentifier)
//  
//  Gives the state of the eight digital inputs(bits) as a decimal value (0-255)
//
//	inDeviceIdentifier: the DeviceIdentifier that identifies the device that's targeted
//
//  Returns the value of the input port.
// ---------------------------------------------------------------------------------------------------------------
UInt16 ServiceUSB_GetInputValue(ServiceUSB_DeviceIdentifierRef inDeviceIdentifier __DefaultParam(kServiceUSB_UseDefaultDeviceIdentifier));

// ---------------------------------------------------------------------------------------------------------------
//  UInt16 ServiceUSB_GetEXValue(ServiceUSB_DeviceIdentifierRef inDeviceIdentifier=kServiceUSB_UseDefaultDeviceIdentifier)
//  
//  Use this function to get the value from analog input EX
//
//	inDeviceIdentifier: the DeviceIdentifier that identifies the device that's targeted
//
//  Returns the value of the ex port.
// ---------------------------------------------------------------------------------------------------------------
UInt16 ServiceUSB_GetEXValue(ServiceUSB_DeviceIdentifierRef inDeviceIdentifier __DefaultParam(kServiceUSB_UseDefaultDeviceIdentifier));

// ---------------------------------------------------------------------------------------------------------------
//  UInt16 ServiceUSB_GetEYValue(ServiceUSB_DeviceIdentifierRef inDeviceIdentifier=kServiceUSB_UseDefaultDeviceIdentifier)
//  
//  Use this function to get the value from analog input EY
//
//	inDeviceIdentifier: the DeviceIdentifier that identifies the device that's targeted
//
//  Returns the value of the ey port.
// ---------------------------------------------------------------------------------------------------------------
UInt16 ServiceUSB_GetEYValue(ServiceUSB_DeviceIdentifierRef inDeviceIdentifier __DefaultParam(kServiceUSB_UseDefaultDeviceIdentifier));

// ---------------------------------------------------------------------------------------------------------------
//  UInt16 ServiceUSB_GetOutputValue(ServiceUSB_DeviceIdentifierRef inDeviceIdentifier=kServiceUSB_UseDefaultDeviceIdentifier)
//  
//  Use this function to get the state of the eight digital outputs(bits) as a decimal value (0-255). The value given corresponds to the value set previously using "ServiceUSB_SetOutputValue"
//
//	inDeviceIdentifier: the DeviceIdentifier that identifies the device that's targeted
//
//  Returns the value of the output port.
// ---------------------------------------------------------------------------------------------------------------
UInt16 ServiceUSB_GetOutputValue(ServiceUSB_DeviceIdentifierRef inDeviceIdentifier __DefaultParam(kServiceUSB_UseDefaultDeviceIdentifier));

// ---------------------------------------------------------------------------------------------------------------
//  OSStatus ServiceUSB_SendVendorRequest(UInt8 inVendorRequest,UInt16 inValue,UInt16 inIndex,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier=kServiceUSB_UseDefaultDeviceIdentifier,UInt32 inTimeout=kServiceUSB_DefaultTimeout)
//  
//  Use this function to send a USB vender specific request to the device. The action/behaviour of the device is USB firmware specific.
//  Currenlty there are no Vendor requests defined.
//
// 	inVendorRequest: the vender request to send
//	inValue: the value for this vendor request
//  inIndex: the index for this vendor request
//	inDeviceIdentifier: the DeviceIdentifier that identifies the device that's targeted
//	inTimeout: the time after which the call will return if the request could not be passed to the device.
//
//  Returns noErr if successfull.
// ---------------------------------------------------------------------------------------------------------------
OSStatus ServiceUSB_SendVendorRequest(UInt8 inVendorRequest,UInt16 inValue,UInt16 inIndex,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier __DefaultParam(kServiceUSB_UseDefaultDeviceIdentifier),UInt32 inTimeout __DefaultParam(kServiceUSB_DefaultTimeout));

// ---------------------------------------------------------------------------------------------------------------
//  UInt32 ServiceUSB_GetBytesInReadBuffer(UInt8 inPipeIndexZeroBase,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier=kServiceUSB_UseDefaultDeviceIdentifier)
//
//  This function is for future use.  
//  Use this function to get the number of byte available in the queue secified by inPipeIndexZeroBase the device.
//
// 	inPipeIndexZeroBase: specifies the pipe (=queue) of the device (3 pipes are available 0, 1 and 2)
//	inDeviceIdentifier: the DeviceIdentifier that identifies the device that's targeted
//
//  Returns the number of bytes in the queue specified by inPipeIndexZeroBase.
// ---------------------------------------------------------------------------------------------------------------
UInt32 ServiceUSB_GetBytesInReadBuffer(UInt8 inPipeIndexZeroBase,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier __DefaultParam(kServiceUSB_UseDefaultDeviceIdentifier));

// ---------------------------------------------------------------------------------------------------------------
//  ServiceUSB_ReadBytesFromReadBuffer(UInt8 inPipeIndexZeroBase,void* inBuffer,UInt32* inOutReadBytes,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier=kServiceUSB_UseDefaultDeviceIdentifier)
//
//  This function is for future use.  
//  Use this function to read a specified number of bytes from the queue specified by inPipeIndexZeroBase.
//
// 	inPipeIndexZeroBase: specifies the pipe (=queue) of the device (3 pipes are available 0, 1 and 2)
//  inBuffer: a pointer to the buffer that will contain the data from the queue on return.
//						Make sure the buffer is big enough to hold the number of bytes specified in inOutReadBytes
// 	inOutReadBytes: On input this the number of maximum bytes to read.
//									On output this the number of bytes actually returned in inBuffer
//	inDeviceIdentifier: the DeviceIdentifier that identifies the device that's targeted
//
//  Returns the number of bytes in the queue specified by inPipeIndexZeroBase.
// ---------------------------------------------------------------------------------------------------------------
void 	ServiceUSB_ReadBytesFromReadBuffer(UInt8 inPipeIndexZeroBase,void* inBuffer,UInt32* inOutReadBytes,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier __DefaultParam(kServiceUSB_UseDefaultDeviceIdentifier));

// ---------------------------------------------------------------------------------------------------------------
//  OSStatus ServiceUSB_WriteBytes(UInt8 inPipeIndexZeroBase,void* inData,UInt32* inOutDataLen,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier=kServiceUSB_UseDefaultDeviceIdentifier,UInt32 inTimeout=kServiceUSB_DefaultTimeout)
//
//  This function is for future use.  
//  Use this function to write a specific number of bytes to a queue specified by inPipeIndexZeroBase.
//
// 	inPipeIndexZeroBase: specifies the pipe (=queue) of the device (3 pipes are available 0, 1 and 2)
//  inData: a pointer to the buffer that contain the data to be written to the queue.
// 	inOutDataLen: On input this the number of byte to be written from inData.
//								On output this the number of bytes actually be written to the device
//	inDeviceIdentifier: the DeviceIdentifier that identifies the device that's targeted
//	inTimeout: the time after which the call will return if the request could not be passed to the device.
//
//  Returns noErr if successfull.
// ---------------------------------------------------------------------------------------------------------------
OSStatus ServiceUSB_WriteBytes(UInt8 inPipeIndexZeroBase,void* inData,UInt32* inOutDataLen,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier __DefaultParam(kServiceUSB_UseDefaultDeviceIdentifier),UInt32 inTimeout __DefaultParam(kServiceUSB_DefaultTimeout));

// ---------------------------------------------------------------------------------------------------------------
//  void ServiceUSB_ReleaseDevice(ServiceUSB_DeviceIdentifierRef inDeviceIdentifier=kServiceUSB_UseDefaultDeviceIdentifier);
//  
//  Use this function to release the device so it can be used in other application. This function is only need for Mac OS X.
//	On Mac OS 9 this function does nothing.
//
//	inDeviceIdentifier: the DeviceIdentifier that identifies the device that's targeted
// ---------------------------------------------------------------------------------------------------------------
void ServiceUSB_ReleaseDevice(ServiceUSB_DeviceIdentifierRef inDeviceIdentifier __DefaultParam(kServiceUSB_UseDefaultDeviceIdentifier));

// ---------------------------------------------------------------------------------------------------------------
// ---------------------------------------------------------------------------------------------------------------
// --------------------- API functions to support special task of the Fischertechnik Interface -------------------
// ---------------------------------------------------------------------------------------------------------------
// ---------------------------------------------------------------------------------------------------------------


#define kServiceUSB_8BitMaskStringLength			8											// Maximum bits we support on 8 bit devices
#define kServiceUSB_BitMaskStringLength				8											// deprecated; use kServiceUSB_BitMask16BitStringLength
#define kServiceUSB_16BitMaskStringLength			16										// Maximum bits we support on 16 bit devices			
#define kServiceUSB_EngineCount								4											// Maximum engines we support

// a bit mask is defined as a series of chars
// 'I' means set the bit of the value
// '1' means set the bit of the value
// 'O' means clear the bit of the value
// '0' means clear the bit of the value
// 'T' means invert the bit of the value
// a typical string would be "OIIOOIOO" where the highest bit is the character from the left
// in the example above the value resulting from the bits above would be 0x64
typedef char ServiceUSB_8BitMaskStringType[kServiceUSB_8BitMaskStringLength];							// type that holds the BitMaskInfo
typedef char ServiceUSB_BitMaskStringType[kServiceUSB_BitMaskStringLength];								// deprecated; use ServiceUSB_16BitMaskStringType
typedef char ServiceUSB_16BitMaskStringType[kServiceUSB_16BitMaskStringLength];						// type that holds the BitMaskInfo

typedef SInt16 ServiceUSB_EngineStatusType[kServiceUSB_EngineCount];											// type that holds the EngineStatus Info
typedef UInt16 ServiceUSB_InputValueType[kServiceUSB_EngineCount];												// type that holds the InputValues for the Engine

// ---------------------------------------------------------------------------------------------------------------
//  OSStatus ServiceUSB_SetOutputValueByMask(const ServiceUSB_BitMaskStringType& inBitMask,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier=kServiceUSB_UseDefaultDeviceIdentifier,UInt32 inTimeout=kServiceUSB_DefaultTimeout)
//  
//  Use this function to set the output value according the mask supplied.
//  This function first reads the current value of the output and then sets the bits
//
// 	inBitMask: specifies the bit mask for the output value (e.g. "10IOOITX")
//	inDeviceIdentifier: the DeviceIdentifier that identifies the device that's targeted
//	inTimeout: the time after which the call will return if the request could not be passed to the device.
//
//  Returns noErr if successfull.
// ---------------------------------------------------------------------------------------------------------------
#ifdef __cplusplus
OSStatus ServiceUSB_SetOutputValueBy16BitMask(const ServiceUSB_16BitMaskStringType& inBitMask,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier __DefaultParam(kServiceUSB_UseDefaultDeviceIdentifier),UInt32 inTimeout __DefaultParam(kServiceUSB_DefaultTimeout));
OSStatus ServiceUSB_SetOutputValueBy8BitMask(const ServiceUSB_8BitMaskStringType& inBitMask,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier __DefaultParam(kServiceUSB_UseDefaultDeviceIdentifier),UInt32 inTimeout __DefaultParam(kServiceUSB_DefaultTimeout));
// deprecated; use ServiceUSB_SetOutputValueBy8BitMask
OSStatus ServiceUSB_SetOutputValueByMask(const ServiceUSB_8BitMaskStringType& inBitMask,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier __DefaultParam(kServiceUSB_UseDefaultDeviceIdentifier),UInt32 inTimeout __DefaultParam(kServiceUSB_DefaultTimeout)) DEPRECATED_ATTRIBUTE;
#else
OSStatus ServiceUSB_SetOutputValueBy16BitMask(const ServiceUSB_16BitMaskStringType* inBitMask,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier __DefaultParam(kServiceUSB_UseDefaultDeviceIdentifier),UInt32 inTimeout __DefaultParam(kServiceUSB_DefaultTimeout));
OSStatus ServiceUSB_SetOutputValueBy8BitMask(const ServiceUSB_8BitMaskStringType* inBitMask,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier __DefaultParam(kServiceUSB_UseDefaultDeviceIdentifier),UInt32 inTimeout __DefaultParam(kServiceUSB_DefaultTimeout));
// deprecated; use ServiceUSB_SetOutputValueBy8BitMask
OSStatus ServiceUSB_SetOutputValueByMask(const ServiceUSB_8BitMaskStringType* inBitMask,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier __DefaultParam(kServiceUSB_UseDefaultDeviceIdentifier),UInt32 inTimeout __DefaultParam(kServiceUSB_DefaultTimeout)) DEPRECATED_ATTRIBUTE;
#endif // __cplusplus

// ---------------------------------------------------------------------------------------------------------------
//  void ServiceUSB_GetInputValueAsMask(ServiceUSB_BitMaskStringType outBitMask,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier=kServiceUSB_UseDefaultDeviceIdentifier)
//  
//  Use this function to get the input value as a BitMask.
//
// 	outBitMask: input value return as a mask (e.g. "OIIOOIOO")
//							'I' = bit set
//							'O' = bit not set
//	inDeviceIdentifier: the DeviceIdentifier that identifies the device that's targeted
// ---------------------------------------------------------------------------------------------------------------
void ServiceUSB_GetInputValueAs16BitMask(ServiceUSB_16BitMaskStringType outBitMask,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier __DefaultParam(kServiceUSB_UseDefaultDeviceIdentifier));
void ServiceUSB_GetInputValueAs8BitMask(ServiceUSB_8BitMaskStringType outBitMask,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier __DefaultParam(kServiceUSB_UseDefaultDeviceIdentifier));
// deprecated; use ServiceUSB_GetInputValueAs16BitMask
void ServiceUSB_GetInputValueAsMask(ServiceUSB_BitMaskStringType outBitMask,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier __DefaultParam(kServiceUSB_UseDefaultDeviceIdentifier)) DEPRECATED_ATTRIBUTE;

// ---------------------------------------------------------------------------------------------------------------
//  OSStatus ServiceUSB_SetOutputValueBit(char inSymbol,UInt8 inBit,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier=kServiceUSB_UseDefaultDeviceIdentifier,UInt32 inTimeout=kServiceUSB_DefaultTimeout)
//  
//  Use this function manipulate one bit of the output value, specified in inSymbol.
//  This function first reads the current value of the output and then sets the bit
//
// 	inSymbol: how to set the bit
//	inBit: which bit to set (0..7)
//	inDeviceIdentifier: the DeviceIdentifier that identifies the device that's targeted
//	inTimeout: the time after which the call will return if the request could not be passed to the device.
//
//  Returns noErr if successfull.
// ---------------------------------------------------------------------------------------------------------------
OSStatus ServiceUSB_SetOutputValueBit(char inSymbol,UInt8 inBit,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier __DefaultParam(kServiceUSB_UseDefaultDeviceIdentifier),UInt32 inTimeout __DefaultParam(kServiceUSB_DefaultTimeout));

// ---------------------------------------------------------------------------------------------------------------
//  char GetInputValueBit(UInt8 inBit,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier=kServiceUSB_UseDefaultDeviceIdentifier)
//  
//  Use this function to find out if a bit is set of not in the input value.
//
// 	inBit: the bit of the input value the information is requested (0..7)
//	inDeviceIdentifier: the DeviceIdentifier that identifies the device that's targeted
//
//  Returns the mask for the bit specified in inBit.
//	'I' = bit set
//	'O' = bit not set
// ---------------------------------------------------------------------------------------------------------------
char ServiceUSB_GetInputValueBit(UInt8 inBit,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier __DefaultParam(kServiceUSB_UseDefaultDeviceIdentifier));

// ---------------------------------------------------------------------------------------------------------------
//  Boolean ServiceUSB_TestInputValueBit(UInt8 inBit,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier=kServiceUSB_UseDefaultDeviceIdentifier)
//  
//  Use this function to find out if a bit is set of not in the input value.
//
// 	inBit: the bit of the input value to test
//	inDeviceIdentifier: the DeviceIdentifier that identifies the device that's targeted
//
//  Returns true if the bit is set.
// ---------------------------------------------------------------------------------------------------------------
Boolean ServiceUSB_TestInputValueBit(UInt8 inBit,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier __DefaultParam(kServiceUSB_UseDefaultDeviceIdentifier));

// ---------------------------------------------------------------------------------------------------------------
//  Boolean ServiceUSB_CompareInputValueMask(const ServiceUSB_BitMaskStringType& inBitMask,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier=kServiceUSB_UseDefaultDeviceIdentifier)
//  
//  Use this function to compare the input value against a given BitMask.
//
// 	inBitMask: the BitMask to test against (e.g. "10IOOIXX")
//						 'I' = bit set
//						 'O' = bit not set
//	inDeviceIdentifier: the DeviceIdentifier that identifies the device that's targeted
//
//  Returns true if the bit mask matches the input value.
// ---------------------------------------------------------------------------------------------------------------
#ifdef __cplusplus
Boolean ServiceUSB_CompareInputValue16BitMask(const ServiceUSB_16BitMaskStringType& inBitMask,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier __DefaultParam(kServiceUSB_UseDefaultDeviceIdentifier));
Boolean ServiceUSB_CompareInputValue8BitMask(const ServiceUSB_8BitMaskStringType& inBitMask,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier __DefaultParam(kServiceUSB_UseDefaultDeviceIdentifier));
// deprecated; use ServiceUSB_GetInputValueAs16BitMask
Boolean ServiceUSB_CompareInputValueMask(const ServiceUSB_BitMaskStringType& inBitMask,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier __DefaultParam(kServiceUSB_UseDefaultDeviceIdentifier)) DEPRECATED_ATTRIBUTE;
#else
Boolean ServiceUSB_CompareInputValue16BitMask(const ServiceUSB_16BitMaskStringType* inBitMask,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier __DefaultParam(kServiceUSB_UseDefaultDeviceIdentifier));
Boolean ServiceUSB_CompareInputValue8BitMask(const ServiceUSB_8BitMaskStringType* inBitMask,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier __DefaultParam(kServiceUSB_UseDefaultDeviceIdentifier));
// deprecated; use ServiceUSB_GetInputValueAs16BitMask
Boolean ServiceUSB_CompareInputValueMask(const ServiceUSB_BitMaskStringType* inBitMask,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier __DefaultParam(kServiceUSB_UseDefaultDeviceIdentifier)) DEPRECATED_ATTRIBUTE;
#endif // __cplusplus

// ---------------------------------------------------------------------------------------------------------------
//  OSStatus ServiceUSB_PulseEngineWhileMouseButton(ServiceUSB_EngineStatusType inOutEngineStatus,const ServiceUSB_InputValueType& inInputValueBit,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier=kServiceUSB_UseDefaultDeviceIdentifier,UInt32 inTimeout=kServiceUSB_DefaultTimeout)
//  
//  This function manually deduces the number of pulses necessary for the next command (ServiceUSB_PulseEngine)
//  The motor rotates for as long as the mouse button is pressed.
//  inOutEngineStatus:	Negative number: Motor is rotating left
//  										Positive number: Motor is rotating right
//  										Zero: State of the motor remains unchanged
//  inInputValueBit: indicate the respective key inputs for motors 1 to 4. The values range from 0 and 7 and refer to inputs E0 to E7.
//	inDeviceIdentifier: the DeviceIdentifier that identifies the device that's targeted
//	inTimeout: the time after which the call will return if the request could not be passed to the device.
//
//  Returns noErr if successfull.
// ---------------------------------------------------------------------------------------------------------------
#ifdef __cplusplus
OSStatus ServiceUSB_PulseEngineWhileMouseButton(ServiceUSB_EngineStatusType inOutEngineStatus,const ServiceUSB_InputValueType& inInputValueBit,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier __DefaultParam(kServiceUSB_UseDefaultDeviceIdentifier),UInt32 inTimeout __DefaultParam(kServiceUSB_DefaultTimeout));
#else
OSStatus ServiceUSB_PulseEngineWhileMouseButton(ServiceUSB_EngineStatusType inOutEngineStatus,const ServiceUSB_InputValueType* inInputValueBit,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier __DefaultParam(kServiceUSB_UseDefaultDeviceIdentifier),UInt32 inTimeout __DefaultParam(kServiceUSB_DefaultTimeout));
#endif // __cplusplus

// ---------------------------------------------------------------------------------------------------------------
//  OSStatus ServiceUSB_PulseEngine(const ServiceUSB_EngineStatusType& inEngineStatus,const ServiceUSB_InputValueType& inInputValueBit,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier=kServiceUSB_UseDefaultDeviceIdentifier,UInt32 inTimeout=kServiceUSB_DefaultTimeout)
//  
//  This command is mainly used for the motor control unit of certain fischertechnik models. The motors M1 to M4 must each be connected to two outputs for use
//  M1 between A0 and A1 (Motor 1 to output 0 and output 1),M2 between A2 and A3, M3 between A4 and A5, M4 between A6 and A7
//  The motor will continue to rotate after the command has been sent until a certain number of feedback pulses have been detected
//  Feedback is given via the inputs E7 to E0, at which keys that emit pulses when the motor is rotating are normally connected.
//
//  inOutEngineStatus:	Negative number: Motor is rotating left
//  										Positive number: Motor is rotating right
//  										Zero: State of the motor remains unchanged
//  inInputValueBit: indicate the respective key inputs for motors 1 to 4. The values range from 0 and 7 and refer to inputs E0 to E7.
//	inDeviceIdentifier: the DeviceIdentifier that identifies the device that's targeted
//	inTimeout: the time after which the call will return if the request could not be passed to the device.
//
//  Returns noErr if successfull.
// ---------------------------------------------------------------------------------------------------------------
#ifdef __cplusplus
OSStatus ServiceUSB_PulseEngine(const ServiceUSB_EngineStatusType& inEngineStatus,const ServiceUSB_InputValueType& inInputValueBit,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier __DefaultParam(kServiceUSB_UseDefaultDeviceIdentifier),UInt32 inTimeout __DefaultParam(kServiceUSB_DefaultTimeout));
#else
OSStatus ServiceUSB_PulseEngine(const ServiceUSB_EngineStatusType* inEngineStatus,const ServiceUSB_InputValueType* inInputValueBit,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier __DefaultParam(kServiceUSB_UseDefaultDeviceIdentifier),UInt32 inTimeout __DefaultParam(kServiceUSB_DefaultTimeout));
#endif // __cplusplus

// ---------------------------------------------------------------------------------------------------------------
//  OSStatus RegisterNotification(UInt32 inNotificationType,void* inNotificationData,ServiceUSB_NotificationProc inNotificationProc,void *inUserRefCon,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier=kServiceUSB_UseDefaultDeviceIdentifier)
//  
//  Use this function to allow to receive a notification through the notification proc if a certain values has changed or reached the trigger point.
//	Only one notification can be active at a point.
//	This call is only supported on Mac OS X
//
//  inNotificationType: the notification kind you want to use
//  inNotificationData: the trigger data (eServiceUSB_InputValueMaskNotification: a ServiceUSB_BitMaskStringType, all others are UInt8)
//	inUserRefCon: any user data that is passed back to through the notification proc specfied
//	inDeviceIdentifier: the DeviceIdentifier that identifies the device that's targeted
//
//  Returns noErr if successfull.
// ---------------------------------------------------------------------------------------------------------------

// ---------------------------------------------------------------------------------------------------------------
//  void void (*ServiceUSB_NotificationProc)(UInt32 inNotificationType,UInt16 inComparisonValue,void *userRefCon)
//  
//  This is the function called when the notifcation is triggered
// 
// 	inNotificationType: The notification passed when ServiceUSB_RegisterNotification was called
//	inComparisonValue: The value that triggered the notification
//	userRefCon: The userRefCon passed when ServiceUSB_RegisterNotification was called
// ---------------------------------------------------------------------------------------------------------------

enum
{
	eServiceUSB_NoNotification=0,										// no notification will be generated
	eServiceUSB_InputValueMaskNotification=1,				// Notification Data ServiceUSB_8BitMaskStringType
	eServiceUSB_InputValue8BitMaskNotification=1,		// Notification Data ServiceUSB_8BitMaskStringType
	eServiceUSB_EXValueGreaterNotification,					// Notification Data is data to compare
	eServiceUSB_EXValueLessNotification,						// Notification Data is data to compare
	eServiceUSB_EYValueGreaterNotification,					// Notification Data is data to compare
	eServiceUSB_EYValueLessNotification,						// Notification Data is data to compare
	eServiceUSB_BytesInReadPipeNotification,				// Notification Data is read pipe index 0, 1, 2
	eServiceUSB_InputValue16BitMaskNotification			// Notification Data ServiceUSB_16BitMaskStringType
};

typedef void (*ServiceUSB_NotificationProc)(UInt32 inNotificationType,UInt16 inComparisonValue,void *userRefCon);

OSStatus ServiceUSB_RegisterNotification(UInt32 inNotificationType,void* inNotificationData,ServiceUSB_NotificationProc inNotificationProc,void *inUserRefCon,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier __DefaultParam(kServiceUSB_UseDefaultDeviceIdentifier));

// ---------------------------------------------------------------------------------------------------------------
// ---------------------------------------------------------------------------------------------------------------
// ----------------------------- API functions to get general information about devices --------------------------
// ---------------------------------------------------------------------------------------------------------------
// ---------------------------------------------------------------------------------------------------------------

// ---------------------------------------------------------------------------------------------------------------
//  UInt16 ServiceUSB_GetDeviceCount(void)
//  
//  Use this function to get the number of connected SERVICE USB devices.
//  
//  Returns the number of devices
// ---------------------------------------------------------------------------------------------------------------
UInt16 ServiceUSB_GetDeviceCount(void);

// ---------------------------------------------------------------------------------------------------------------
//  ServiceUSB_DeviceIdentifierRef GetNextDeviceIdentifier(ServiceUSB_DeviceIdentifierRef inLastDeviceIdentifer)
//  
//  Use this function iterate through all connected devices.
//
//  inLastDeviceIdentifer: The DeviceIdentifier returned from the last call to GetNextDeviceIdentifier.
//											 	 When call is issued for the first time to receive the first device in the chain pass
//											 	 kServiceUSB_UseDefaultDeviceIdentifier.
//
//  Returns the number of devices
// ---------------------------------------------------------------------------------------------------------------
ServiceUSB_DeviceIdentifierRef ServiceUSB_GetNextDeviceIdentifier(ServiceUSB_DeviceIdentifierRef inLastDeviceIdentifer);

// ---------------------------------------------------------------------------------------------------------------
//  OSStatus ServiceUSB_GetDeviceSerialnumber(Str255 outSerialNr,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier=kServiceUSB_UseDefaultDeviceIdentifier)
//  
//  Use this function to get the serialnr of the device.
//
//  outSerialNr: The serial as a string
//	inDeviceIdentifier: the DeviceIdentifier that identifies the device that's targeted
//
//  Returns noErr if successfull.
// ---------------------------------------------------------------------------------------------------------------
OSStatus ServiceUSB_GetDeviceSerialnumber(Str255 outSerialNr,ServiceUSB_DeviceIdentifierRef inDeviceIdentifier __DefaultParam(kServiceUSB_UseDefaultDeviceIdentifier));

// ---------------------------------------------------------------------------------------------------------------
//  UInt16 ServiceUSB_GetDeviceROMVersion(ServiceUSB_DeviceIdentifierRef inDeviceIdentifier=kServiceUSB_UseDefaultDeviceIdentifier)
//  
//  Use this function to get the ROM version of the device.
//
//	inDeviceIdentifier: the DeviceIdentifier that identifies the device that's targeted
//
//  Returns the version number of the device in BCD format (e.g. 0x0100). Returns 0x0000 in case of an error.
// ---------------------------------------------------------------------------------------------------------------
UInt16 ServiceUSB_GetDeviceROMVersion(ServiceUSB_DeviceIdentifierRef inDeviceIdentifier __DefaultParam(kServiceUSB_UseDefaultDeviceIdentifier));

// ---------------------------------------------------------------------------------------------------------------
//  UInt32 ServiceUSB_GetUSBDriverVersion(ServiceUSB_DeviceIdentifierRef inDeviceIdentifier=kServiceUSB_UseDefaultDeviceIdentifier)
//  
//  Use this function to get USB driver software version for this device.
//
//	inDeviceIdentifier: the DeviceIdentifier that identifies the device that's targeted
//
//  Returns the version number of the USB driver software in BCD format (e.g. 0x01008000). Returns 0x0000 in case of an error.
// ---------------------------------------------------------------------------------------------------------------
UInt32 ServiceUSB_GetUSBDriverVersion(ServiceUSB_DeviceIdentifierRef inDeviceIdentifier __DefaultParam(kServiceUSB_UseDefaultDeviceIdentifier));

// ---------------------------------------------------------------------------------------------------------------
//  Boolean ServiceUSB_Is16BitDevice(ServiceUSB_DeviceIdentifierRef inDeviceIdentifier=kServiceUSB_UseDefaultDeviceIdentifier)
//  
//  Use this function to find out if the device support 16 bit values.
//
//	inDeviceIdentifier: the DeviceIdentifier that identifies the device that's targeted
//
//  Returns true if the device is a 16 bit device.
// ---------------------------------------------------------------------------------------------------------------
Boolean ServiceUSB_Is16BitDevice(ServiceUSB_DeviceIdentifierRef inDeviceIdentifier __DefaultParam(kServiceUSB_UseDefaultDeviceIdentifier));

// ---------------------------------------------------------------------------------------------------------------
//  ServiceUSB_Version2String(const NumVersion& inData,Str255 outResult)
//  
//  This is a utility function that allows you to convert the version number returned from GetUSBDriverVersion into a string.
//
//	inData: a version number e.g. returned from GetUSBDriverVersion
//	outResult: the version as a string (e.g. 1.0)
// ---------------------------------------------------------------------------------------------------------------
#ifdef __cplusplus
void ServiceUSB_Version2String(const NumVersion& inData,Str255 outResult);
#else
void ServiceUSB_Version2String(const NumVersion* inData,Str255 outResult);
#endif // __cplusplus

#ifdef __cplusplus
}
#endif

#endif // __ServiceUSBToolbox__